/*
** conf.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ipc.h> 
#include <sys/shm.h> 

#include "conf.h"
#include "spy.h"
#include "vshlib.h"


char *confs[NB_CONF] =
  {
    NULL,
    "bind",
    "port",
    "$include",
    "log",
    "addrule",
    "backlog",
    "enable_filter",
    "disable_filter",
    "enable_log_filter",
    "disable_log_filter"
  };

conf_node_t   *lstconf = NULL;

extern spy_filter_t tabfilter_header [];
extern spy_filter_content_t tabfilter_content [];

/*
 * Private Interface
 */


/* Add a node of configuration configuration*/
static void conf_add_node(conf_node_type type, char *value)
{
  conf_node_t *new_node, *tmp_node;

  new_node = vsh_malloc(sizeof(conf_node_t));
  new_node->type = type;
  new_node->value = strdup(value);
  new_node->next = NULL;

  if (!lstconf)
    {
      lstconf = new_node;
      return ;
    }
  for (tmp_node = lstconf; tmp_node->next; tmp_node = tmp_node->next) ;
  tmp_node->next = new_node;
}

/* Process "addrule" directive */
static void conf_add_rule(char *line)
{
  int i, j;
  char *buf_filter;
  vsh_str_t str_filter;

  /* filter extraction */
  vsh_str_init(&str_filter);
  for (i = 0; line[i] && isblank(line[i]); i++);
  for (i; line[i] && !isblank(line[i]); i++)
    vsh_str_addchar(&str_filter, line[i]);
  buf_filter = vsh_str_inline(&str_filter);

  /* call of appropriate configuration fct */
  if (buf_filter && line[i])
    {
      for (j = 0; tabfilter_header[j].name; j++)
	if (!strcmp(buf_filter, tabfilter_header[j].name))
	  if (tabfilter_header[j].fct_filter_policy_rule != NULL)
	    {
	      tabfilter_header[j].fct_filter_policy_rule(&line[i + 1]);
	      vsh_str_free(&str_filter);
	      vsh_free(buf_filter);
	      return ;
	    }
      
      for (j = 0; tabfilter_content[j].name; j++)
	if (!strcmp(buf_filter, tabfilter_content[j].name))
	  if (tabfilter_content[j].fct_filter_policy_rule != NULL)
	    {
	      tabfilter_content[j].fct_filter_policy_rule(&line[i + 1]);
	      vsh_str_free(&str_filter);
	      vsh_free(buf_filter);
	      return ;
	    }
    }
  perror("conf_add_rule");
  exit(1);
}


/* Process "enable filter" directive */
static void conf_enable_filter(char *filter_name)
{
  int i;
  for (i = 0; tabfilter_header[i].name; i++)
    if (!strcmp(filter_name, tabfilter_header[i].name))
      {
	tabfilter_header[i].active = 1;
	return ;
      }
  for (i = 0; tabfilter_content[i].name; i++)
    if (!strcmp(filter_name, tabfilter_content[i].name))
      {
	tabfilter_content[i].active = 1;
	return ;
      }
  perror("conf_enable_filter");
  exit(1);
}

/* Process "disable filter" directive */
static void conf_disable_filter(char *filter_name)
{
  int i;

  for (i = 0; tabfilter_header[i].name; i++)
    if (!strcmp(filter_name, tabfilter_header[i].name))
      {
	tabfilter_header[i].active = 0;
	return ;
      }
  for (i = 0; tabfilter_content[i].name; i++)
    if (!strcmp(filter_name, tabfilter_content[i].name))
      {
	tabfilter_content[i].active = 0;
	return ;
      }
  perror("conf_disable_filter");
  exit(1);
}

/* Process "enable_log filter" directive */
static void conf_enable_filter_log(char *filter_name)
{
  int i;
  for (i = 0; tabfilter_header[i].name; i++)
    if (!strcmp(filter_name, tabfilter_header[i].name))
      {
	tabfilter_header[i].active_log = 1;
	return ;
      }
  for (i = 0; tabfilter_content[i].name; i++)
    if (!strcmp(filter_name, tabfilter_content[i].name))
      {
	tabfilter_content[i].active_log = 1;
	return ;
      }
  perror("conf_enable_log_filter");
  exit(1);
}

/* Process "disable_log filter" directive */
static void conf_disable_filter_log(char *filter_name)
{
  int i;

  for (i = 0; tabfilter_header[i].name; i++)
    if (!strcmp(filter_name, tabfilter_header[i].name))
      {
	tabfilter_header[i].active_log = 0;
	return ;
      }
  for (i = 0; tabfilter_content[i].name; i++)
    if (!strcmp(filter_name, tabfilter_content[i].name))
      {
	tabfilter_content[i].active_log = 0;
	return ;
      }
  perror("conf_disable_log_filter");
  exit(1);
}

/*Parse a line of configuration file */
static int conf_parse_line(char *line)
{
  int         i, j, res;
  vsh_str_t   str_order;
  char       *buf_order, c;
  conf_node_type type;

  /* command extraction */
  for (i = 0; line[i] && isblank(line[i]); i++) ;

  vsh_str_init(&str_order);
  for (i; line[i] && !isblank(line[i]); i++)
    vsh_str_addchar(&str_order, tolower(line[i]));
  buf_order = vsh_str_inline(&str_order);
  vsh_str_free (&str_order);

  if (!buf_order)
    return ;

  for (j = 1; j < NB_CONF; j++)
    if (!strcmp(buf_order, confs[j]))
      {
	type = (conf_node_type)j;
	break ;
      }
  vsh_free(buf_order);
  if (j == NB_CONF)
    return 1;
    
  /* command processing */
  switch(type)
    {
    case CONF_UNKNOW:
      perror("unknow conf parameter");
      exit(1);
      break ;

    case CONF_ADD_RULE:
      conf_add_rule(&line[i]);
      return 0;
      break;

    case CONF_ENABLE_FILTER:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_enable_filter(buf_order);
      vsh_free(buf_order);
      return 0;
      break;

    case CONF_DISABLE_FILTER:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_disable_filter(buf_order);
      vsh_free(buf_order);
      return 0;
      break;

    case CONF_ENABLE_LOG_FILTER:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_enable_filter_log(buf_order);
      vsh_free(buf_order);
      return 0;
      break;
      
    case CONF_DISABLE_LOG_FILTER:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_disable_filter_log(buf_order);
      vsh_free(buf_order);
      return 0;
      break;

    case CONF_INCLUDE:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      res = conf_parse_file(buf_order);
      vsh_free(buf_order);
      if (!res)
	return 1;
      return 0;
      break;

      // the blank are used
    case CONF_LOG:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i]; i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_add_node(type, buf_order);
      vsh_free(buf_order);
      return 0;
      break;

    default:
      vsh_str_init(&str_order);
      for (i; line[i] && isblank(line[i]); i++);
      for (i; line[i] && !isblank(line[i]); i++)
	vsh_str_addchar(&str_order, line[i]);
      buf_order = vsh_str_inline(&str_order);
      conf_add_node(type, buf_order);
      vsh_free(buf_order);
      return 0;
      break;
    }

  return 1;
}


/*
 * Public Interface
 */


/* Main primitive of configuration file processing */
int conf_parse_file(char *filename)
{
  char       c;
  int        fd, res, stop = 0;
  vsh_str_t  str_line;
  char      *buf_line;
  
  fd = vsh_open(filename, O_RDONLY, 1);

  do {
      /* line extraction */
      vsh_str_init(&str_line);
      c = 0;
      res = vsh_read(fd, &c, 1);
      while (res > 0 && c != '\n' && c != '#')
	{
	  vsh_str_addchar(&str_line, c);
	  res = vsh_read(fd, &c, 1);
	}
      
      /* line parsing */
      if (str_line.size > 0)
	{
	  buf_line = vsh_str_inline(&str_line);
	  stop = conf_parse_line(buf_line);
	  vsh_free(buf_line);
	}
      else
	if (res <= 0)
	  stop = 1 ;
      
      if (!stop && c == '#')
	{
	  do {
	    res = vsh_read(fd, &c, 1);
	  }
	  while (res > 0 && c != '\n');
	}
      vsh_str_free(&str_line);
    }while (!stop && res > 0);

  return 1;
}


/* Extract a configuration node */
char *conf_get_node (conf_node_type node_type)
{
  conf_node_t *tmp_node;

  for (tmp_node = lstconf; tmp_node; tmp_node = tmp_node->next)
    if (tmp_node->type == node_type)
      return tmp_node->value;
  return NULL;
}

/* Configuration checking */
void conf_debug()
{
  int i;
  conf_node_t *tmp_node;

  vsh_log(FAC_INFO, "-- Configuration");
  vsh_log(FAC_INFO, "-- Nodes");
  for (tmp_node = lstconf; tmp_node; tmp_node = tmp_node->next)
    {
      vsh_log(FAC_INFO, confs[(int)(tmp_node->type)]);
      vsh_log(FAC_INFO, tmp_node->value);
    }
  vsh_log (FAC_INFO, "-- Configuration");
}
