/*
** http_parse.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <ctype.h>

#include "vshlib.h"
#include "tokenlib.h"
#include "spy.h"

#include "http.h"
#include "http_methods.h"

/* 
 * Private Interface 
 */

extern char *methods  [NB_METHOD];
extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];


static void http_parse_req_method(token_session_t *session, http_req_t *req)
{
  int j;
  char *buffer;

  buffer = token_get(session, 0);
  
  if (!buffer)
    {
      vsh_log(FAC_INFO, "unknow request type ...");
      spy_die();
    }

  // Search the method in the reference list 
  for (j = 0; j < NB_METHOD; j++)
    {
      if (!strcmp(buffer, methods[j]))
	{
	  vsh_log(FAC_INFO, buffer);
	  req->req = (req_type)j;
	  break ;
	}
    }
  if (j == NB_METHOD)
    {
      vsh_log(FAC_INFO, "unknow receved request type ...");
      spy_die();
    }

  vsh_free(buffer);
}

static void http_parse_req_url(token_session_t *session, http_req_t *req)
{
  /* default setting */
  req->port  = HTTP_STD_PORT;
  req->https = 0;
  req->url   = token_get(session, 1);

  if (!req->url)
    spy_die();

  vsh_log(FAC_INFO, req->url);
}

static void http_parse_req_version(token_session_t *session, http_req_t *req)
{
  int i;
  char *version;

  version = token_get(session, 0);
  
  if (!version)
    spy_die();

  for (i = 0; i < NB_VERSION; i++)
    if (!strcmp(version, versions[i]))
      {
	req->version = (http_version)i;
	break ;
      }

  if (i == NB_VERSION)
    spy_die();

  vsh_free(version);
}


static void http_parse_req_add_fields(http_req_t *req, char *name, char *value)
{
  int i;
  field_type type = FLD_UNKNOW;

  if (!value)
    return ;
  
  /* Search of header type */
  for (i = 0; i < NB_HEADER; i++)
    if (!strcmp(name, headers[i]))
      {
	type = (field_type)i;
	break ;
      }
  req->lst_fields = http_methods_add_field(req->lst_fields, type, value);
}


static void http_parse_req_fields(token_session_t *session, http_req_t *req)
{
  int i;
  char *buffer_field_name;
  char *buffer_field_value;
  

  while (1)
    {
      buffer_field_name = token_get(session, 0);
      if (!buffer_field_name)
	break ;
      
      buffer_field_value = token_get_line(session);
      if (!buffer_field_value)
       spy_die();

      for (i = 0; i < strlen(buffer_field_name); i++)
	buffer_field_name[i] = tolower(buffer_field_name[i]);
      for (i = 0; i < strlen(buffer_field_value); i++)
	buffer_field_value[i] = buffer_field_value[i];

      http_parse_req_add_fields(req, buffer_field_name, buffer_field_value);
    }
}


static void http_parse_req_body   (http_req_t *req)
{

  char *lenght;

  lenght = http_method_search_field(req->lst_fields, FLD_CONTENT_LENGHT);
  
  if (lenght)
    req->body_lenght = atoi(lenght);
  else
    req->body_lenght = 0;

  // req->body = token_get_stream(session, &(req->body_lenght));
}


/* 
 * Public Interface 
 */


/*Main Primitive of HTTP request parsing*/

token_session_t *session_cli;
void http_parse_req(int sock, http_req_t *req)
{
  session_cli = token_init(sock);

  http_parse_req_method(session_cli, req);
  http_parse_req_url(session_cli, req);
  http_parse_req_version(session_cli, req);
  http_parse_req_fields(session_cli, req);

  if (req->req == REQ_PUT || req->req == REQ_POST)
    http_parse_req_body(req);
  else
    {
      req->body = NULL;
      req->body_lenght = -1;
    }
}
