/*
** http_parse_resp.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "vshlib.h"
#include "tokenlib.h"

#include "http.h"
#include "http_methods.h"

extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];


/* 
 * Private Interface 
 */


static void http_parse_resp_version (token_session_t *session, http_resp_t *resp)
{
  int i;
  char *version;

  version = token_get(session, 0);

  if (!version)
    spy_die();

  for (i = 0; i < NB_VERSION; i++)
    if (!strcmp(version, versions[i]))
      {
	resp->version = (http_version)i;
	break ;
      }

  if (i == NB_VERSION)
    spy_die();

  vsh_free(version);
}


static void http_parse_resp_status (token_session_t *session, http_resp_t *resp)
{
  char *status;
  
  status = token_get(session, 0);
  
  if(!status)
    spy_die();

  resp->status = atoi(status);
  vsh_free(status);  
  status = token_get_line(session);
  vsh_free(status);
}


static void http_parse_resp_add_fields(http_resp_t *resp, char *name, char *value)
{
  int i;
  field_type type = FLD_UNKNOW;

  if (!value)
    return ;

  /* Search of header type */
  for (i = 0; i < NB_HEADER; i++)
    if (!strcmp(name, headers[i]))
      {
	type = (field_type)i;
	break ;
      }
  resp->lst_fields = http_methods_add_field(resp->lst_fields, type, value);
}


static void http_parse_resp_fields(token_session_t *session, http_resp_t *resp)
{
  int i;
  char *buffer_field_name;
  char *buffer_field_value;
  

  while (1)
    {
      buffer_field_name = token_get(session, 0);
      if (!buffer_field_name)
	break ;
      buffer_field_value = token_get_line(session);

      if(!buffer_field_value)
	spy_die();

      for (i = 0; i < strlen(buffer_field_name); i++)
	buffer_field_name[i] = tolower(buffer_field_name[i]);
      for (i = 0; i < strlen(buffer_field_value); i++)
	buffer_field_value[i] = tolower(buffer_field_value[i]);

      http_parse_resp_add_fields(resp, buffer_field_name, buffer_field_value);
    }
}


/* 
 * Public Interface 
 */



/* Main Primitive of HTTP response headers parsing */

token_session_t *session_srv;
void http_parse_resp_headers(int sock, http_resp_t *resp)
{

  session_srv = token_init(sock);

  http_parse_resp_version (session_srv, resp);
  http_parse_resp_status  (session_srv, resp);
  http_parse_resp_fields  (session_srv, resp);
}
