/*
** http_transmit.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>

#include "spy.h"
#include "vshlib.h"
#include "tokenlib.h"

#include "mime.h"
#include "http.h"
#include "http_methods.h"


extern char *methods  [NB_METHOD];
extern char *headers  [NB_HEADER];
extern char *versions [NB_VERSION];

extern http_session_t http_session;

extern token_session_t *session_srv;
extern token_session_t *session_cli;
extern spy_filter_content_t tabfilter_content [];


static int http_transmit_connect_srv(http_req_t *req)
{
  int i;
  struct timeval tval;
  int sock_srv; 
  char *host;
  struct hostent *he;
  struct sockaddr_in addr_srv;

  host = strdup(http_method_search_field(req->lst_fields, FLD_HOST));
  for (i = 0; host[i]; i++)
    if (host[i] == ':')
      {
	host[i] = '\0';
	break ;
      }

  // FIXME : gerer le fait que le host soit une adresse IP ...
  if ((he = gethostbyname(host)) == NULL) 
    {
      vsh_free(host);
      return -1;
    }
  vsh_free(host);
  sock_srv = vsh_socket(AF_INET, SOCK_STREAM, 0);

  addr_srv.sin_family = AF_INET;
  addr_srv.sin_port = htons(req->port);
  addr_srv.sin_addr = *((struct in_addr *)he->h_addr);
  memset(&(addr_srv.sin_zero), '\0', 8);
  
  tval.tv_sec = 10;
  tval.tv_usec = 0;
  setsockopt(sock_srv, SOL_SOCKET, SO_RCVTIMEO, &tval, sizeof(struct timeval));
  
  if (vsh_connect(sock_srv, (struct sockaddr *)&addr_srv, sizeof(struct sockaddr)) < 0)
    return -2;
  
  memcpy(&(http_session.addr_srv), &addr_srv, sizeof(struct sockaddr_in));
  return sock_srv;
}



void http_transmit_req(int *sock_srv, int sock_cli, http_req_t *req)
{
  char *buf, *content_type;
  int size, i, total_size = 0;
  unsigned char *buffer_flush, buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  http_field_t *tmp_field;
  vsh_str_t str_req;
  
  content_type = http_method_search_field(req->lst_fields, FLD_CONTENT_TYPE);

  // HTTP server connection
  if (http_session.first_connect || !http_session.keep_alive)
    *sock_srv = http_transmit_connect_srv(req);
  
  // inpossible to connect
  if (*sock_srv < 0)
    return ;

  // not https processing here
  if (req->https)
    return ;

  // send the request
  //   -> cmd
  vsh_str_init(&str_req);
  for (i = 0; methods[(int)req->req][i]; i++)
    vsh_str_addchar(&str_req, methods[(int)req->req][i]);
  vsh_str_addchar(&str_req, ' ');

  for (i = 0; req->url[i]; i++)
    vsh_str_addchar(&str_req, req->url[i]);
  vsh_str_addchar(&str_req, ' ');

  for (i = 0; versions[(int)req->version][i]; i++)
    vsh_str_addchar(&str_req, versions[(int)req->version][i]);
  vsh_str_addchar(&str_req, '\r');
  vsh_str_addchar(&str_req, '\n');
  
  buf = vsh_str_inline(&str_req);
  vsh_send(*sock_srv, buf, strlen(buf), 0);


  // -> headers
  for (tmp_field = req->lst_fields; tmp_field; tmp_field = tmp_field->next)
    {
      vsh_send(*sock_srv, headers[(int)tmp_field->type], 
	       strlen(headers[(int)tmp_field->type]), 0);
      vsh_send(*sock_srv, ": ", 2, 0);
      vsh_send(*sock_srv, tmp_field->value, strlen(tmp_field->value), 0);
      vsh_send(*sock_srv, "\r\n", 2, 0);
    }
  vsh_send(*sock_srv, "\r\n", 2, 0);

  vsh_str_free(&str_req);
  vsh_free(buf);

  // -> body
  // NEW FEAT pas d'envoi en une fois sinon probleme
  //if (req->body_lenght)
  //  vsh_send(*sock_srv, req->body, req->body_lenght, 0);

  if (req->body_lenght < 0)
    return ;

  /* body content sending*/
  buffer_flush = token_get_flush(session_cli, &size);
  if (size)
    {
      for (i = 0; tabfilter_content[i].name; i++)
	if (tabfilter_content[i].active)
	  if (tabfilter_content[i].fct_filter(buffer_flush, size, content_type, tabfilter_content[i].active_log) == 0)
	    return ;
      
      vsh_send(*sock_srv, buffer_flush, size, 0);
    }
  
  total_size += size;
  if (req->body_lenght && total_size >= req->body_lenght)
    return ;
  
  do
    {
      size = HTTP_TRANSMIT_BUFFER_SIZE;
      size = vsh_recv(sock_cli, buffer, size, 0);
      
      // on-the-fly filtering
      if (size > 0)
	{
	  for (i = 0; tabfilter_content[i].name; i++)
	    if (tabfilter_content[i].active)
	      if (tabfilter_content[i].fct_filter(buffer, size, content_type, tabfilter_content[i].active_log) == 0)
		return ;
	  vsh_send(*sock_srv, buffer, size, 0);
	}
      
      total_size += size;
      if (req->body_lenght && total_size >= req->body_lenght)
	break ;
      
    }while (size > 0);
}


void http_transmit_resp(int sock_cli, int sock_srv, http_resp_t *resp)
{
  int           i, size, lenght, total_size = 0;
  unsigned char *buffer_flush, buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  char          *buf_lenght, *content_type;
  char           status[5];
  http_field_t  *tmp_field;
  

  content_type = http_method_search_field(resp->lst_fields, FLD_CONTENT_TYPE);

  
  // --> send status
  vsh_send(sock_cli, versions[(int)resp->version], 
	   strlen(versions[(int)resp->version]), 0);
  vsh_send(sock_cli, " ", 1, 0);

  if (resp->status < 0 || resp->status > 999)
    spy_die();
  sprintf(status, "%d ", resp->status);
  status[4] = '\0';
  vsh_send(sock_cli, status, strlen(status), 0);

  vsh_send(sock_cli, "\r\n", 2, 0);
  

  // --> send HTTP headers
  for (tmp_field = resp->lst_fields; tmp_field; tmp_field = tmp_field->next)
    {
      vsh_send(sock_cli, headers[(int)tmp_field->type], 
	       strlen(headers[(int)tmp_field->type]), 0);
      vsh_send(sock_cli, ": ", 2, 0);
      vsh_send(sock_cli, tmp_field->value, strlen(tmp_field->value), 0);
      vsh_send(sock_cli, "\r\n", 2, 0);
    }
  vsh_send(sock_cli, "\r\n", 2, 0);


  // -> send body
  buf_lenght = http_method_search_field(resp->lst_fields, FLD_CONTENT_LENGHT);
  if (buf_lenght)
    lenght = atoi(buf_lenght);
  else
    lenght = 0;

  /* cf : pb de apache mod_proxy */
  if (lenght < 0)
    return ;

  if (sock_srv <= 0)
    {
      /* error sending */
      if (resp->body_lenght)
	vsh_send(sock_cli, resp->body, resp->body_lenght, 0);
    }
  else
    {
      /* body content sending*/
      buffer_flush = token_get_flush(session_srv, &size);
      if (size)
	{
	  for (i = 0; tabfilter_content[i].name; i++)
	    if (tabfilter_content[i].active)
	      if (tabfilter_content[i].fct_filter(buffer_flush, size, content_type, tabfilter_content[i].active_log) == 0)
		return ;
	      
	  vsh_send(sock_cli, buffer_flush, size, 0);
	}

      total_size += size;
      if (lenght && total_size >= lenght)
	return ;
      
      do
	{
	  size = HTTP_TRANSMIT_BUFFER_SIZE;
	  size = vsh_recv(sock_srv, buffer, size, 0);

	  // on-the-fly filtering
	  if (size > 0)
	    {
	      for (i = 0; tabfilter_content[i].name; i++)
		if (tabfilter_content[i].active)
		  if (tabfilter_content[i].fct_filter(buffer, size, content_type, tabfilter_content[i].active_log) == 0)
		    return ;
	      vsh_send(sock_cli, buffer, size, 0);
	    }

	  total_size += size;
	  if (lenght && total_size >= lenght)
	    break ;
	  
	}while (size > 0);
    }
}
