/*
** http_transmit_https.c
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <fcntl.h>

#include "http.h"
#include "vshlib.h"
#include "tokenlib.h"

/*
 * Private Interface
 */

static void http_transmit_https_ok(int sock_cli, token_session_t *session_srv)
{
  http_resp_t resp;

  http_methods_init_resp(&resp);

  resp.version = DEFAULT_HTTP_VERSION;
  resp.status = 200;
 
  resp.lst_fields = http_methods_add_field(resp.lst_fields, FLD_PROXY_CONNECTION, "keep-alive");
  resp.lst_fields = http_methods_add_field(resp.lst_fields, FLD_KEEP_ALIVE, "5");

  resp.body = NULL;
  resp.body_lenght  = 0;

  http_transmit_resp(sock_cli, 0, &resp);
}


/*
 * Public interface
 */

void http_transmit_https(int sock_srv, int sock_cli)
{
  struct timeval tv;

  fd_set set;   // master file descriptor list
  int fdmax;    // maximum file descriptor number
  int more;

  unsigned char buffer[HTTP_TRANSMIT_BUFFER_SIZE];
  int   lenght, res;
  token_session_t *session_srv, *session_cli;
  
  session_srv = token_init(sock_srv);
  session_cli = token_init(sock_cli);

  http_transmit_https_ok(sock_cli, session_srv);
  
 http_transmit_https_start:  

  FD_ZERO(&set);

  // add the listener to the master set
  FD_SET(sock_srv, &set);
  FD_SET(sock_cli, &set);

  tv.tv_sec = 2;
  //tv.tv_usec = 500000;
  tv.tv_usec = 0;

  if (sock_srv < sock_cli)
    fdmax = sock_cli;
  else
    fdmax = sock_srv;

  more = 0;
  select(fdmax + 1, &set, NULL, NULL, &tv);
  
  if (FD_ISSET(sock_cli, &set))
    {
      res = vsh_recv(sock_cli, &buffer, HTTP_TRANSMIT_BUFFER_SIZE, 0);
      if (res > 0)
	{
	  //printf("flip [%c]\n", buffer);
	  res = vsh_send(sock_srv, &buffer, res, 0);
	  if (res > 0 -1)
	    more = 1;
	}
    }
  if (FD_ISSET(sock_srv, &set))
    {
      res = vsh_recv(sock_srv, &buffer, HTTP_TRANSMIT_BUFFER_SIZE, 0);
      if (res > 0)
	{
	  //printf("flop [%c]\n", buffer);
	  res = vsh_send(sock_cli, &buffer, res, 0);
	  if (res > 0)
	    more = 1;
	}
    }
  if (more)
    goto http_transmit_https_start;
}
