/*
** spy.h
** Login : <julien@localhost>
** $Id$
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	SPY_H_
# define   	SPY_H_

#include "http.h"


/*
 * Public Interface
 */

#define POLICY_BLOCK_ALL 0
#define POLICY_ALLOW_ALL 1


/* Signature Definition*/

typedef enum 
{
  SIGN_ACT_BLOCK,
  SIGN_ACT_SHAPE
}sign_action;

struct spy_sign_t
{
  char *url;
  char *fld;
  char *msg;
  sign_action action;
  struct spy_sign_t *next;
};
typedef struct spy_sign_t spy_sign_t;

/* Keyword Definition, usefull for matching */

struct spy_key_t
{
  char *key;
  struct spy_key_t *next;
};
typedef struct spy_key_t spy_key_t;

/* HTTP Header Filter Definition */

struct spy_filter_t
{
  char *name;
  int   active;
  int   active_log;
  int   (*fct_filter_req)(http_req_t *req, int log);
  int   (*fct_filter_resp)(http_resp_t *resp, int log);
  int   (*fct_filter_policy_rule)(char *rule);
  int   (*fct_filter_init)();
};
typedef struct spy_filter_t spy_filter_t;

/* HTTP content Filter Definition */

struct spy_filter_content_t
{
  char *name;
  int   active;
  int   active_log;
  int   (*fct_filter)(void *data, int lenght, char *type, int log);
  int   (*fct_filter_policy_rule)(char *rule);
  int   (*fct_filter_init)();
};
typedef struct spy_filter_content_t spy_filter_content_t;


/* Signature filter definition */

int  spy_sign_init ();
int  spy_sign_policy_rule (char *rule);
int  spy_sign_req  (http_req_t *req, int log);
int  spy_sign_resp (http_resp_t *resp, int log);

/* URL filter definition */

int  spy_block_req_url             (http_req_t *req, int log);
int  spy_block_url_policy_rule (char *rule);
int  spy_block_url_init();

/* Content filter definition */

int  spy_block_content  (void *data, int lenght, char *type, int log);
int  spy_block_content_policy_rule (char *rule);
int  spy_block_content_init();

/* Type file filter definition*/

int  spy_block_req_type             (http_req_t *req, int log);
int  spy_block_resp_type            (http_resp_t *resp, int log);
int  spy_block_type_policy_rule (char *rule);
int  spy_block_type_init();

/* Shellcode Filter definition  */

int  spy_shape_req_shellcode             (http_req_t *req, int log);
int  spy_shape_resp_shellcode            (http_resp_t *resp, int log);
int  spy_shape_shellcode_policy_rule (char *rule);
int  spy_shape_shellcode_init();

/* Cookie Filter definition */

int  spy_shape_req_cookie             (http_req_t *req, int log);
int  spy_shape_resp_cookie            (http_resp_t *resp, int log);
int  spy_shape_cookie_policy_rule (char *rule);
int  spy_shape_cookie_init();

/* Identity Filter definition */

int spy_shape_req_identity             (http_req_t *req, int log);
int spy_shape_resp_identity            (http_resp_t *resp, int log);
int spy_shape_identity_policy_rule (char *rule);
int spy_shape_identity_init();

/* Popup Filter definition */

int spy_shape_popup (void *data, int lenght, char *type, int log);
int spy_shape_popup_policy_rule (char *rule);
int spy_shape_popup_init();

/*Standard HTTP Error Response */

void spy_response_block       (http_resp_t *resp);
void spy_response_unreach     (http_resp_t *resp);
void spy_response_unconnect   (http_resp_t *resp);


/* Filter tools */

spy_key_t *spy_key_add(spy_key_t *keys, char *keyword);
int        spy_key_fnmatch(spy_key_t *keys, char *test);


/*
 * Main Filter Primitives
 */

int spy_filter_req(http_req_t *req_cli, http_req_t *req_srv);
int spy_filter_resp(http_resp_t *resp_srv, http_resp_t *resp_cli);

void spy_core (int sock_cli);
void spy_log(char *filtername, char *log_format, int is_req);
void spy_die();

#endif 	    /* !SPY_H_ */
