/*
** spy_filter_sign.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include "spy.h"
#include "conf.h"
#include "vshlib.h"

spy_sign_t *tabsign = NULL;


static void spy_sign_addsign(char *url, char *fld, char *msg, sign_action action)
{
  spy_sign_t *new_sign, *tmp_sign;

  if (!url && !fld && !msg)
    return ;
  new_sign = vsh_malloc(sizeof(spy_sign_t));
  new_sign->next = NULL;
  
  new_sign->url = NULL;
  new_sign->fld = NULL;
  new_sign->msg = NULL;
  new_sign->action = action;

  if (url)
    new_sign->url = strdup(url);
  if (fld)
    new_sign->fld = strdup(fld);
  if (msg)
    new_sign->msg = strdup(msg);

  // Ajout en tete
  if (tabsign == NULL)
    {
      tabsign = new_sign;
      return ;
    }
  
  // Ajout en queue
  for (tmp_sign = tabsign; tmp_sign->next; tmp_sign= tmp_sign->next) ;
  tmp_sign->next = new_sign;
}


int  spy_sign_init ()
{
  
  return 1;
}


int  spy_sign_policy_rule (char *rule)
{
  int i, ok;
  vsh_str_t str;
  char *content;

  char *url = NULL;
  char *fld = NULL;
  char *msg = NULL;

  i = 0;
  while(i < strlen(rule))
    {
      for (i; rule[i] && isblank(rule[i]); i++);
      if (!rule[i])
	continue ;

      vsh_str_init(&str);
      while (rule[i] && !isblank(rule[i]) && rule[i] != '"')
	{
	  vsh_str_addchar(&str, rule[i]);
	  i++;
	}
      content = vsh_str_inline(&str);
      vsh_str_free(&str);

      while (rule[i] && rule[i] != '"')
	i++ ;
      
      if (!content || !rule[i])
	{
	  perror("spy_sign_policy_rule");
	  exit(1);
	}

      vsh_str_init(&str);

      if (rule[i] && rule[i] == '"')
	i++;
      else
	return 0;

      while (rule[i] && rule[i] != '"')
	{
	  vsh_str_addchar(&str, rule[i]);
	  i++;
	}

      ok = 0;
      if (!strcmp("url", content))
	{
	  url = vsh_str_inline(&str);
	  ok = 1;
	}
      if (!strcmp("fld", content))
	{
	  fld = vsh_str_inline(&str);
	  ok = 1;
	}

      if (!strcmp("msg", content))
	{
	  msg = vsh_str_inline(&str);
	  ok = 1;
	}
      vsh_str_free(&str);
      if (rule[i])
	i++;

      if (!ok)
	break ;
    }
  spy_sign_addsign(url, fld, msg, SIGN_ACT_BLOCK);
  return 1;
}


int  spy_sign_req  (http_req_t *req, int log)
{
  http_field_t *tmp_field;
  spy_sign_t   *tmp_sign;

  for (tmp_sign = tabsign; tmp_sign; tmp_sign = tmp_sign->next)
    {
      /*url signature filtergin*/
      if (tmp_sign->url)
	{
	  if (strstr(req->url, tmp_sign->url) != NULL)
	    {
	      if (tmp_sign->msg && log)
		spy_log("BLOCK_SIGN", tmp_sign->msg, 1);
	      return 0;
	    }
	}

      /*header filtering*/
      if (tmp_sign->fld)
	for (tmp_field = req->lst_fields; tmp_field; tmp_field = tmp_field->next)
	  if (strstr(tmp_field->value, tmp_sign->fld))
	    {
	      if (tmp_sign->msg && log)
		spy_log("BLOCK_SIGN", tmp_sign->msg, 1);
	      return 0;
	    }
    }
  return 1;
}


int  spy_sign_resp (http_resp_t *resp, int log)
{
  http_field_t *tmp_field;
  spy_sign_t   *tmp_sign;

  /* HTTP header signature's filtering */
    for (tmp_sign = tabsign; tmp_sign; tmp_sign = tmp_sign->next)
      if (tmp_sign->fld)
	for (tmp_field = resp->lst_fields; tmp_field; tmp_field = tmp_field->next)
	  if (strstr(tmp_field->value, tmp_sign->fld))
	    {
	      if (tmp_sign->msg && log)
		spy_log("BLOCK_SIGN", tmp_sign->msg, 0);
	      return 0;
	    }
    return 1;
}
