/*
** tokenlib.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "tokenlib.h"
#include "vshlib.h"

/*
 * Private Interface
 */

static int token_getbyte(token_session_t *session, void *c)
{
  if (session->cursor == LIBSESSION_BUFFER_SIZE || session->cursor >= session->size)
    {
      session->size = vsh_read(session->fd, session->buffer, LIBSESSION_BUFFER_SIZE);
      if (session->size <= 0)
	return 0;
      session->cursor = 0;
    }

  *((unsigned char*)c) = session->buffer[session->cursor];
  session->cursor++;

  return 1;
}


static int token_is_separator(char c)
{
  if (c == '\n')
    return 1;
  if (c == '\r')
    return 1;
  return 0;
}


/*
 * Public Interface
 */

token_session_t *token_init(int fd)
{
  token_session_t *session;

  session = (token_session_t *)vsh_malloc(sizeof(token_session_t));
  session->fd = fd;
  session->size = 0;  
  session->cursor = LIBSESSION_BUFFER_SIZE;
  
  return session;
}

char               *token_get(token_session_t *session, char accept_dot)
{
  int res;
  char c;
  char *token;
  vsh_str_t str_token;

  vsh_str_init(&str_token);
  while((res = token_getbyte(session, &c)) > 0)
    {
      if (c == ':')
	continue ;
      else
	if (!isblank(c))
	  break;
    }
  
  if (!res)
    return NULL ;

  do {
    if (!isblank(c)  && !token_is_separator(c))
      {
	if (accept_dot)
	  vsh_str_addchar (&str_token, c);
	else
	  {
	    if (c != ':')
	      vsh_str_addchar (&str_token, c);
	    else
		break ;
	  }
      }
    else
      break ;
    res = token_getbyte(session, &c);
  }while(res > 0);


  if (res && c == '\r')
    token_getbyte(session, &c);

  
  token = vsh_str_inline (&str_token);
  vsh_str_free(&str_token);
  return token;
}

char               *token_get_line(token_session_t *session)
{
  int res;
  char c;
  char *line;
  vsh_str_t str_line;

  vsh_str_init(&str_line);
  while((res = token_getbyte(session, &c)) > 0)
    if (!isblank(c))
      break;

   if (res == 0)
     return NULL;

   do {
     if (!token_is_separator(c))
       vsh_str_addchar (&str_line, c);
     else
       break ;
     res = token_getbyte(session, &c);
   }while(res > 0) ;

    if (res && c == '\r')
      token_getbyte(session, &c);

    line = vsh_str_inline (&str_line);
    vsh_str_free(&str_line);
    return line;
}

unsigned char      *token_get_stream(token_session_t *session, int *lenght)
{
  int i;
  char c;
  unsigned char *stream;
  vsh_str_t  str_stream;
  vsh_char_t *char_stream;

  
  vsh_str_init(&str_stream);
  i = 0;
  while(token_getbyte(session, &c))
    {
      vsh_str_addchar (&str_stream, c);
      i++;
      
      if (*lenght && i >= *lenght)
	break ;
    }
  
  if (*lenght == 0)
    *lenght = str_stream.size;
  
  if (str_stream.size == 0)
    return NULL;
  
  stream = (unsigned char *)vsh_malloc(str_stream.size);
  
  for (i = 0, char_stream = str_stream.head; 
       char_stream; 
       char_stream = char_stream->next, i++)
    stream[i] = char_stream->c;
  
  vsh_str_free(&str_stream);
  return stream;
}


unsigned char *token_get_flush(token_session_t *session, int *lenght)
{
  *lenght = session->size - session->cursor;
  if (*lenght > 0)
    return &(session->buffer[session->cursor]);

  *lenght = 0;
  return NULL;
}

