/*
** vsh_tcp.c
** 
** Copyright Julien Perez (C) 2005 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "vshlib.h"

#include <stdlib.h>
#include <stdio.h>

#include <errno.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>


int vsh_socket(int domain, int type, int protocol)
{
  int sock;

  sock = socket(domain, type, protocol);
  if (sock == -1)
    {
      perror("socket");
      exit(1);
    }
  return sock;
}

int vsh_setsockopt(int s, int level, int optname, const void *optval, socklen_t optlen)
{
  int res;
  
  res = setsockopt(s, level, optname, optval, optlen);
  if (res == -1)
    {
      perror("setsockopt");
      exit(1);
    }
  return res;
}

int vsh_bind(int sockfd, const struct sockaddr *my_addr, socklen_t addrlen)
{
  int res;

  res = bind(sockfd, my_addr, addrlen);
  if (res == -1)
    {
      perror("bind");
      exit(1);
    }
  return res;
}

int vsh_listen(int sockfd, int backlog)
{
  int res;

  res = listen(sockfd, backlog);
  if (res == -1)
    {
      perror("listen");
      exit(1);
    }
  return res;
}

int vsh_accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen)
{
  int new_sock;

  new_sock = accept(sockfd, addr, addrlen);
  if (new_sock == -1)
    {
      perror("accept");
      vsh_close(sockfd);
      exit(1);
    }
  return new_sock;
}

int vsh_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
{
  int res;

  res = connect(sockfd, serv_addr, addrlen);
  if (res < 0)
    {
      perror("connect");
      shutdown(sockfd, SHUT_RDWR);
      vsh_close(sockfd);
    }
  return res;
}

ssize_t vsh_send(int s, const void *buf, size_t len, int flags)
{
  ssize_t size;
  
  size = send(s, buf, len, flags);
  if (size == -1)
    {
      vsh_close(s);
      exit(1);
    }
  return size;
}

ssize_t vsh_recv(int s, void *buf, size_t len, int flags)
{
  ssize_t size;
  
  size = recv(s, buf, len, flags);
  if (size == -1)
    {
      // perror("recv");
      // vsh_close(s);
      // exit(1);
    }
  return size;
}

int vsh_shutdown(int s, int how)
{
  int res;
  
  if (s <= 0)
    return 0;

  res = shutdown(s, how);
  if (res == -1)
    {
      perror("shutdown");
      vsh_close(s);
      //exit(1);
    }
  return res;
}
